#include <ESP8266WiFi.h>
#include <PubSubClient.h>

EthernetClient ethClient;
PubSubClient client(ethClient);
#define MSG_BUFFER_SIZE	(50)
char mqtt_topic[MSG_BUFFER_SIZE];


const char* ssid = "........";
const char* password = "........";
const char* mqtt_server = ".........";


//название устройства в rules
String mqtt_danfos_device_name = "DANFOS_ESP";

void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }



//будет срабатывать при изменении топика, тут мы будем отслеживать вкл/выкл клапанов 
void callback(char* topic, byte* payload, unsigned int length) {
  
      Serial.print("Message arrived [");
      Serial.print(topic);
      Serial.print("] ");
      String str_topic = String(topic);
      
      String payloadString = "";
            for (int i = 0; i < length; i++) {
                payloadString += (char)payload[i];
            }
     //парсим строку сообщения , определяем какой именно клапан нужно обработать
     if (str_topic.substring(0,30) == "/devices/"+mqtt_danfos_device_name+"/controls/P") {
        String vent_str = str_topic.substring(30,31);
        int vent_idx = vent_str.toInt();
        Serial.println("vent_no " + vent_str + " " +String(payloadString));
      }      

  
 

}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    String clientId = "ESP8266Client-DANFOS";
    //clientId += String(random(0xffff), HEX);
    if (client.connect(clientId.c_str())) {
      Serial.println("connected");
      
      
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void mqtt_init()
{
  
  client.setServer(mqtt_server, 1883);
  

  reconnect();
  for (int i = 1; i < 9; i++) {
          snprintf (mqtt_topic, MSG_BUFFER_SIZE, "/devices/%s/controls/P%d", mqtt_danfos_device_name, i);
          
          Serial.println(mqtt_topic);
          //обнуляем значения включения клапанов
          client.publish(mqtt_topic, "0");
          //подписываемся на топики , чтобы получать события о включении/выключении клапана из WB
          client.subscribe(mqtt_topic);
      }
  client.setCallback(callback);   
  client.loop();

  
}

void mqtt_loop()
{
  if (!client.connected()) {
      reconnect();
     }
  client.loop();
}

void publish_temp(int vent_idx,float temperature)
{
  snprintf (mqtt_topic, MSG_BUFFER_SIZE, "/devices/%s/controls/T%d", mqtt_danfos_device_name, vent_idx);
  char temp_val[10];
  sprintf(temp_val, "%.2f", temperature);
  //обновляем значение температуры в WB
  client.publish(mqtt_topic, temp_val);
}

void setup() {
  setup_wifi();
  mqtt_init();
}

void loop() {
  mqtt_loop();
}

}